/******************************************************************************/
/******************************************************************************/
/* PROJET : 68HC11ETHER - CARTE 68HC11 ETHERNET 802.3 10BaseT                 */
/*									      */
/*            ####### #     #  #####  #######   ###   ######  ######          */
/*            #       ##    # #     # #          #    #     # #     #         */
/*            #       # #   # #       #          #    #     # #     #         */
/*            #####   #  #  #  #####  #####      #    ######  ######          */
/*            #       #   # #       # #          #    #   #   #     #         */
/*            #       #    ## #     # #          #    #    #  #     #         */
/*            ####### #     #  #####  #######   ###   #     # ######          */
/*									      */
/*									      */
/*						Ecole			      */
/*			 			 Nationale		      */
/*			  			  Superieure d'		      */
/*			 			   Electronique d'            */
/*						    Informatique et de 	      */
/*			    			     Radiocommunications de   */
/*			     			      Bordeaux		      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Nom du fichier  : cs8900.c						      */
/*									      */
/* Version et date : 1.0 20/01/00					      */
/*									      */
/* Auteur          : P. Kadionik ENSEIRB				      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Description     :                   					      */
/* Fichier C de la bibliotheque de controle de l'interface Ethernet CS8900    */
/*									      */
/******************************************************************************/
/******************************************************************************/
#include "cs8900.h"


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : out16                  					      */
/* Ecriture de 16 bits a l'adresse : LSB->@, MSB->@+1                         */
/*	entree : int adresse                                                  */
/*               int donnee                                                   */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
/******************************************************************************/
void out16(int adresse, int donnee)
{
  char donnee_l, donnee_h;              /* poids faible, fort        */
  char *ptrl = (char *)adresse;		/* pointeur sur poids faible */
  char *ptrh = (char *)(adresse+1);	/* pointeur sur poids fort   */

  donnee_l = donnee & 0xff;
  donnee_h = (donnee >> 8) & 0xff;

  *ptrl = donnee_l;                     /* octet faible --> $ faible */    
  *ptrh = donnee_h;                     /* octet fort   --> $ fort   */
}

	 			
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : out16Intel               					      */
/* Ecriture de 16 bits a l'adresse : MSB->@, LSB->@+1                         */
/*	entree : int adresse                                                  */
/*               int donnee                                                   */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
/******************************************************************************/
void out16Intel(int adresse, int donnee) {
  char donnee_l, donnee_h;              /* poids faible, fort        */
  char *ptrl = (char *)adresse;		/* pointeur sur poids faible */
  char *ptrh = (char *)(adresse+1);	/* pointeur sur poids fort   */

  donnee_l = donnee & 0xff;
  donnee_h = (donnee >> 8) & 0xff;
  *ptrl = donnee_h;                     /* octet fort --> $ faible   */
  *ptrh = donnee_l;                     /* octet faible   --> $ fort */
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : in16                  					      */
/* Lecture de 16 bits a l'adresse : LSB->@, MSB->@+1                          */
/*	entree : int adresse                                                  */
/*      sortie : int, donnee                                                  */
/*									      */
/******************************************************************************/
/******************************************************************************/
int in16(int adresse) {
  char donnee_l, donnee_h;              /* poids faible, fort        */
  char *ptrl = (char *)adresse;		/* pointeur sur poids faible */
  char *ptrh = (char *)(adresse+1);	/* pointeur sur poids fort   */
  int donnee;

  donnee_l = *ptrl;
  donnee_h = *ptrh;
  donnee = ( ( donnee_h << 8 ) + donnee_l ) & 0xffff;
  return(donnee);
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : in16Intel                					      */
/* Lecture de 16 bits a l'adresse : MSB->@, LSB->@+1                          */
/*	entree : int adresse                                                  */
/*      sortie : int, donnee                                                  */
/*									      */
/******************************************************************************/
/******************************************************************************/
int in16Intel(int adresse) {
  char donnee_l, donnee_h;              /* poids faible, fort        */
  char *ptrh = (char *)(adresse+1);	/* pointeur sur poids fort   */
  char *ptrl = (char *)adresse;		/* pointeur sur poids faible */
  
  int donnee;

  donnee_h = *ptrh;
  donnee_l = *ptrl;
  donnee = ( ( donnee_h << 8 ) + donnee_l ) & 0xffff;
  return(donnee);
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : outReg16                					      */
/* Ecriture dans un registre du CS8900A                                       */
/*	entree : int registre (adresse du registre)                           */
/*               int donnee                                                   */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
/******************************************************************************/
void outReg16(int registre, int donnee) {
  out16( PP_POINTER, registre);
  out16( PP_DATA   , donnee );
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : inReg16                					      */
/* Lecture d'un registre du CS8900A                                           */
/*	entree : int registre (adresse du registre)                           */
/*      sortie : int, donnee                                                  */
/*									      */
/******************************************************************************/
/******************************************************************************/
int inReg16(int registre) {
  int donnee;
  out16(PP_POINTER, registre);
  donnee = in16(PP_DATA);	
  return(donnee);
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : cs8900delay              					      */
/* Temporisation car certaines actions sur le CS8900A ne sont pas immediates  */
/*	entree : int time                                                     */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
/******************************************************************************/
void cs8900delay(int time) {
  int i;
  for(i=0; i<0x1000*time; i++)
    ;
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : cs8900init               					      */
/* Initialisation generale du CS8900A et activation ligne en Tx et Rx         */
/*	entree : int time                                                     */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
/******************************************************************************/
void cs8900init(void) {
  /* Configuration de RxCTL (CRC, toutes $, filtre Hash OK)  */
  outReg16(RxCTL, 0x0183);
  
  /* Configuration du registre TestCTL : (pas de LoopBack)   */
  outReg16(TestCTL, 0x0099);

  /* Configuration LineCTL (10BaseT,SerTxOn,SerRxOn)         */
  outReg16(LineCTL, 0x00D3);
  cs8900delay(25);
     
  /* Configuration de BufCFG                                 */
  outReg16(BufCFG, 0x000B);
  cs8900delay(25);  
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : cs8900reset              					      */
/* Reset soft du CS8900A                                                      */
/*	entree : rien                                                         */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
void cs8900reset(void) { 
  int status;
  /* Bit 6 a 1 du registre SelfCTL en 114h */
  outReg16(SelfCTL, RESET);

  /* INITD passe a 1 a la fin du reset */
  while(1) {
	status = inReg16(SelfST) & INITD;
        if(status)
		break;
  }
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : cs8900TxFrame           					      */
/* Emission d'une trame par le CS8900A (Cf AN181)                             */
/* Toute trame inferieure a 60 octets (sans le CRC) est completee a 60 octets */
/*	entree : int lengthword (taille de la trame en MOTS de 16 bits        */
/*               int *data (pointeur sur le buffer contenant la trame a Tx)   */
/*      sortie : int, etat de la transmission                                 */
/*									      */
/******************************************************************************/
/******************************************************************************/
int cs8900TxFrame(int lengthword, int *data) {
  int i, status;

  /* (a) Ecriture de la commande TxCMD                      */	 		
  out16(PP_TxCMD, 0x00c0);

  /* (b) Ecriture de la longueur de la trame en OCTETS      */
  out16(PP_TxLength , lengthword * 2 );

  /* (c) Lecture de BusST                                   */
  status = inReg16(BusST);
  cs8900delay(25);

  /* (d) Test du Bit Rdy4TxNow: On devrait lire BusST=118h  */
  if (status != 0x0118)
	return(FAILURE);

  /* (e) Copie de la trame entiere de TxBuffer dans le chip */
  /* (f) Ecriture de XX/2 words (mots de 16 bits) en        */
  /* IOBase+0x0 (TX Data Port)                              */

  for (i=0; i<lengthword; i++)
     out16Intel(PP_RxTx, *(data++));      
     
  return(SUCCESS);
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : cs8900RxFrame           					      */
/* Lecture d'une trame recue par le CS8900A (Cf AN181)                        */
/*	entree : int *data (pointeur sur le buffer contenant la trame Rx)     */
/*      sortie : int, longueur de la trame sans son CRC                       */
/*									      */
/******************************************************************************/
/******************************************************************************/
int cs8900RxFrame(int *data) {
  int i, var;
  int sizeofRxFrame;

  /* (a) Lecture de RxEvent ou RxStatus                     */
  var = in16Intel(PP_RxTx);
  
  /* (b) Lecture de Length of Frame                         */
  sizeofRxFrame = in16Intel(PP_RxTx);

  /* (c) Recuperation de la trame entiere en RxBuffer       */  
  for (i=0; i<sizeofRxFrame; i++) {	
	var = in16(PP_RxTx);	   /* !!! Sens de Lecture   */		
	out16((int)&data[i],var);  /* sauvegarde dans buffer*/
	}

  return(sizeofRxFrame);
}


/******************************************************************************/
/******************************************************************************/
/*									      */
/* Fonction : cs8900init4IRQ           					      */
/* Initialisation du CS8900A pour IRQ en reception de trame                   */
/*	entree : rien                                                         */
/*      sortie : rien                                                         */
/*									      */
/******************************************************************************/
/******************************************************************************/
void cs8900init4IRQ(void) {

  /* Configuration BufCFG: (RxDestiE bit F) 800B */
  /* outReg16(BufCFG, 0x800B); */

  /* Configuration RxCFG: (RxOiE)                */
  outReg16(RxCFG, 0x0103);  

  /* Selection de l'interruption IRQ0            */
  outReg16(IRQNbr,0x0000); 

  /* Configuration de BusCTL: (EnableIRQ)        */
  outReg16(BusCTL, 0x8017); 
      
  cs8900delay(25);
}