/******************************************************************************/
/******************************************************************************/
/* PROJET : 68HC11ETHER - CARTE 68HC11 ETHERNET 802.3 10BaseT                 */
/*									      */
/*            ####### #     #  #####  #######   ###   ######  ######          */
/*            #       ##    # #     # #          #    #     # #     #         */
/*            #       # #   # #       #          #    #     # #     #         */
/*            #####   #  #  #  #####  #####      #    ######  ######          */
/*            #       #   # #       # #          #    #   #   #     #         */
/*            #       #    ## #     # #          #    #    #  #     #         */
/*            ####### #     #  #####  #######   ###   #     # ######          */
/*									      */
/*									      */
/*						Ecole			      */
/*			 			 Nationale		      */
/*			  			  Superieure d'		      */
/*			 			   Electronique d'            */
/*						    Informatique et de 	      */
/*			    			     Radiocommunications de   */
/*			     			      Bordeaux		      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Nom du fichier  : cs8900.h						      */
/*									      */
/* Version et date : 1.0 20/01/00					      */
/*									      */
/* Auteur          : P. Kadionik ENSEIRB				      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Description     :                   					      */
/* Fichier include de la bibliotheque de controle de l'interface Ethernet     */
/* CS8900A                                                                    */
/*									      */
/******************************************************************************/
/******************************************************************************/


/******************************************************************************/
/* Definition des ports I/O du CS8900A                                        */
/******************************************************************************/
#define IO_BASE 	0x7300
#define PP_RxTx 	IO_BASE + 0x0000
#define PP_TxCMD	IO_BASE + 0x0004
#define PP_TxLength 	IO_BASE + 0x0006
#define PP_ISQ	 	IO_BASE + 0x0008
#define PP_POINTER 	IO_BASE + 0x000a
#define PP_DATA 	IO_BASE + 0x000c


/******************************************************************************/
/* Definition des Registres d'interface bus                                   */
/******************************************************************************/
#define ProdId		0x0000
#define IOBaseAddr	0x0020
#define IRQNbr 		0x0022
#define DMAChanNbr	0x0024
#define DMASOF		0x0026
#define DMAFrCount	0x0028
#define RxDMACount	0x002a
#define MEMBaseAddr	0x002c


/******************************************************************************/
/* Definition des Registres de controle et de configuration du CS8900A        */
/******************************************************************************/
#define RxCFG		0x0102
#define RxCTL 		0x0104
#define TxCFG		0x0106
#define TxCMD		0x0108
#define BufCFG 		0x010a
#define LineCTL 	0x0112
#define SelfCTL 	0x0114
#define BusCTL		0x0116
#define TestCTL 	0x0118


/******************************************************************************/
/* Definition du registre d'etat ISQ et evenements du CS8900A                 */
/******************************************************************************/
#define ISQ		0x0120
#define RxEvent 	0x0124
#define TxEvent 	0x0128
#define BufEvent 	0x012c
#define RxMISS		0x0130
#define TxCOL		0x0132
#define LineST		0x0134
#define SelfST		0x0136
#define BusST		0x0138
#define TDR		0x013c

#define ISQRegNbrMask	0x003F
#define ISQRegContMask	0xffc0
#define RxEventRegNbr	0x0004
#define TXEventRegNbr	0x0008
#define BufEventRegNbr	0x000c
#define RxMISSRegNbr	0x0010
#define TxColRegNbr	0x0012

/* RxEvent                     */
#define ExtraData	0x4000
#define Runt		0x2000
#define CRCError	0x1000
#define Broadcast	0x0800
#define IndAddr		0x0400
#define Hashed		0x0200
#define RxOK		0x0100
#define DribbleBits	0x0080
#define IAHash		0x0040


/******************************************************************************/
/* Definition diverses                                                        */
/******************************************************************************/
#define SUCCESS 0
#define FAILURE 1


/******************************************************************************/
/* Masque de bits                                                             */
/******************************************************************************/
#define INITD 		0x0080  /* bit 7 */
#define RESET		0x0040  /* bit 4 */
#define TxPadDis	0x2000	/* bit d */

/******************************************************************************/
/* Validation des interruptions du 68HC11                                     */
/******************************************************************************/
#define cli()	_asm("cli\n")


/******************************************************************************/
/* Prototype des fonctions de la librarie libcs8900                           */
/******************************************************************************/
void out16(int adresse,int donnee);	        /* Ecriture 16 bits en $      */
int in16(int adresse);			        /* Lecture 16 bits en $       */
void out16Intel(int adresse,int donnee);        /* Ecriture 16 bits en $      */ 
int in16Intel(int adresse);		        /* Lecture 16 bits en $       */

void outReg16(int registre, int donnee);        /* Ecriture 16 bits en REG    */
int inReg16(int registre);		        /* Lecture 16 bits en REG     */

void cs8900delay(int time);		        /* Temporisation              */
void cs8900init(void);		        	/* Initialisation interface   */
void cs8900reset(void);			        /* Reset software             */

void cs8900IRQproc(void);			/* Traitement IRQ du 68HC11   */

int cs8900TxFrame(int lengthword, int *data);   /* Transmission trame         */
int cs8900RxFrame(int *data);		        /* Reception trame            */
