/******************************************************************************/
/******************************************************************************/
/* PROJET : 68HC11ETHER - CARTE 68HC11 ETHERNET 802.3 10BaseT                 */
/*									      */
/*            ####### #     #  #####  #######   ###   ######  ######          */
/*            #       ##    # #     # #          #    #     # #     #         */
/*            #       # #   # #       #          #    #     # #     #         */
/*            #####   #  #  #  #####  #####      #    ######  ######          */
/*            #       #   # #       # #          #    #   #   #     #         */
/*            #       #    ## #     # #          #    #    #  #     #         */
/*            ####### #     #  #####  #######   ###   #     # ######          */
/*									      */
/*									      */
/*						Ecole			      */
/*			 			 Nationale		      */
/*			  			  Superieure d'		      */
/*			 			   Electronique d'            */
/*						    Informatique et de 	      */
/*			    			     Radiocommunications de   */
/*			     			      Bordeaux		      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Nom du fichier  : tst.c						      */
/*									      */
/* Version et date : 1.0 20/01/00					      */
/*									      */
/* Auteur          : P. Kadionik ENSEIRB				      */
/*									      */
/******************************************************************************/
/******************************************************************************/
/*									      */
/* Description     :                   					      */
/* Fichier de test de la carte 68HC11ETHER avec la bibliotheque libcs8900     */
/*									      */
/*									      */
/* Utilisation du CS8900A de Cirrus Logic en Mode I/O 8 Bits                  */
/*									      */
/* Le CS8900A de la carte a ete adresse dans la zone $7000-$7FFF.             */
/* En Mode I/O 8 bits, les acces sur le 8900A se font a partir des 8 ports    */
/* d'E/S 16 bits.                                                             */
/*									      */
/*  Remarques:                                                                */
/*									      */
/* Les fonctions in16 et in16Intel NE CONCERNENT PAS le CADRAGE de donnees    */
/* mais l'ORDRE DE LECTURE des donnees !!!!!!!                                */
/*									      */
/*									      */
/*  Notes concernant les interrupteurs de la carte :                          */
/*									      */
/*     i/o_mem  : permet de choisir avec/sans entree d'interruption.          */
/*               i/o: SANS entree IRQ du 68HC11                               */
/*	         mem: AVEC entree IRQ du 68HC11                               */
/*									      */
/*    MODB..MODA : permet la selection de la configuration du 68HC11.         */
/*               0  0 : Bootstrap ----> PCBUG11.                              */
/*	         1  1 : EXtended -----> BUFFALO si ROMH sinon autonome.       */
/*									      */
/*    RAMH_ROMH  : permet de choisir soit RAM ou ROM en adressage haut.       */
/******************************************************************************/
/******************************************************************************/
#include "cs8900.h"

#define SIZE_OF_FRAME_TX 1514
char bufTx[SIZE_OF_FRAME_TX];

#define SIZE_OF_FRAME_RX 1514
char bufRx[SIZE_OF_FRAME_RX];


/******************************************************************************/
/* Routine d'interruption IRQ                                                 */
/******************************************************************************/
@interrupt void  cs8900IRQproc(void) {
  int status, reg_nbr, reg_content;
  int *ZoneRx, *length, *nbr;

  while(1) {
  	status = inReg16(ISQ);

  	if (status == 0x0000)    /* tous les evenements sont traites */
		return;

  	/****************************************************/
  	/* Traitement des evenements selon organigramme p80 */
  	/****************************************************/
        reg_nbr = (status & ISQRegNbrMask);
        reg_content = (status & ISQRegContMask);

        /****************************************************/
  	/* RxEvent                                          */
	/****************************************************/
  	if(reg_nbr == RxEventRegNbr) {
		if(reg_content & RxOK) {
			ZoneRx = (int *)0x3000;  
  			length = (int *)0x2000;
  			nbr = (int *)0x2010;

  			*length = cs8900RxFrame(ZoneRx);
  			*nbr += 1;
		}
		if(reg_content & IndAddr) {
		}
		if(reg_content & Hashed) {
		}
		if(reg_content & IAHash) {
		}
		if(reg_content & Broadcast) {
		}
		if(reg_content & Runt) {
		}
		if(reg_content & ExtraData) {
		}
		if(reg_content & CRCError) {
		}
		if(reg_content & DribbleBits) {
		}
  	}

  	/****************************************************/
  	/* TxEvent                                          */
  	/****************************************************/
  	if(reg_nbr == TXEventRegNbr) {
  	}

  	/****************************************************/
  	/* BufEvent                                         */
  	/****************************************************/
  	if(reg_nbr == BufEventRegNbr) {
  	}

  	/****************************************************/
  	/* RxMISS                                           */
  	/****************************************************/
  	if(reg_nbr == RxMISSRegNbr) {
  	}

  	/****************************************************/
  	/* TxColl                                           */
  	/****************************************************/
  	if(reg_nbr == TxColRegNbr) {
  	}
  }
}


/******************************************************************************/
/* Test de transmission de trames                                             */
/******************************************************************************/
void  TestTx(int nombre_frame) {
  int i;

  for(i=1; i<=nombre_frame; i++)
      cs8900TxFrame((sizeof(bufTx))/2, (int *)bufTx);
}


/******************************************************************************/
/* Test de reception de trames en polling                                     */
/******************************************************************************/
void TestRxPoll(void) {

  int *ZoneRx, *length, *nbr;
  int etat;
  int size_of_frame_rx;
 
  /* Scrutation jusqu'a reception : POLLING */
  while ( (etat = inReg16(RxEvent)) != 0x0104) 
	;

  ZoneRx = (int *)0x5000;  
  length = (int *)0x4000;
  nbr = (int *)0x4010;

  *length = cs8900RxFrame(ZoneRx);
  *nbr += 1;
}


/******************************************************************************/
/* Test de reception de trames sous interruption                              */
/******************************************************************************/
void TestRxIRQ(void) {

    cs8900init4IRQ();   /* Validation du CS8900 en IRQ        */

    cli();              /* Activation IRQ 68HC11              */

			/* Tout est fait dans la routine d'IT */ 
    while(1)            /* Wait and see                       */
	;
}


void main() {
 int i;

 cs8900reset();
 cs8900init();

/******************************/
/* TestTx                     */
/******************************/
/*
 for(i=0; i<sizeof(bufTx); i++)
        bufTx[i] = (char)i;
 for(i=1; i<=10; i++)
 TestTx(10000);
*/

/******************************/
/* TestRx                     */
/******************************/
/*
while(1) {
 TestRxPoll();
}
*/

/******************************/
/* TestRxIRQ                  */
/******************************/
 TestRxIRQ();
}
